/*
 * Copyright (c) Contributors, http://opensimulator.org/
 * See CONTRIBUTORS.TXT for a full list of copyright holders.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the OpenSimulator Project nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using Nini.Config;
using log4net;
using System;
using System.Reflection;
using System.IO;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;
using System.Xml.Serialization;
using System.Collections.Generic;
using OpenSim.Server.Base;
using OpenSim.Services.Interfaces;
using FriendInfo = OpenSim.Services.Interfaces.FriendInfo;
using GridRegion = OpenSim.Services.Interfaces.GridRegion;
using OpenSim.Framework;
using OpenSim.Framework.Servers.HttpServer;
using OpenMetaverse;
using OpenSim.Framework.ServiceAuth;

namespace OpenSim.Server.Handlers.Hypergrid
{
    public class NEWHGFriendsServerPostHandler : BaseStreamHandler
    {
        private static readonly ILog m_log = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);

        private IUserAgentService m_UserAgentService;
        private IFriendsSimConnector m_FriendsLocalSimConnector;
        private IHGFriendsService m_TheService;

        private HGFriendsServerPostHandler m_realHandler = null;

        List<String> m_fixCache = new List<String>();

        public NEWHGFriendsServerPostHandler(IHGFriendsService service, IUserAgentService uas, IFriendsSimConnector friendsConn) : base("POST", "/hgfriends")
        {
            Console.WriteLine("----------- LOADING NEWHGFriendsServerPostHandler -----------");
            m_TheService = service;
            m_UserAgentService = uas;
            m_FriendsLocalSimConnector = friendsConn;
            m_realHandler = new HGFriendsServerPostHandler(service, uas, friendsConn);

            if (File.Exists("friends.txt"))
                m_fixCache = new List<String>(File.ReadAllLines("friends.txt"));
        }

        public static Stream GenerateStreamFromString(string s)
        {
            MemoryStream stream = new MemoryStream();
            StreamWriter writer = new StreamWriter(stream);
            writer.Write(s);
            writer.Flush();
            stream.Position = 0;
            return stream;
        }
        
        protected override byte[] ProcessRequest(string path, Stream requestData, IOSHttpRequest httpRequest, IOSHttpResponse httpResponse)
        {
            try
            {
                using (StreamReader _streamReader = new StreamReader(requestData))
                {
                    String body = _streamReader.ReadToEnd();
                    //m_log.Info("BODY: " + body);

                    Dictionary<string, object> request = ServerUtils.ParseQueryString(body);

                    if (request.ContainsKey("userID"))
                    {
                        string userID = request["userID"].ToString();

                        if (request.ContainsKey("METHOD"))
                        {
                            if(request["METHOD"].ToString() == "statusnotification")
                            {
                                for (int _friendID = 0; request.ContainsKey("friend_" + _friendID); _friendID++)
                                {
                                    String FriendData = userID + ";" + request["friend_" + _friendID].ToString().Trim();

                                    if (!m_fixCache.Contains(FriendData))
                                        m_fixCache.Add(FriendData);
                                }
                            }

                            if (request["METHOD"].ToString() == "getfullfriendsfile")
                            {
                                httpResponse.StatusCode = (int)200;
                                httpResponse.ContentType = "text/plain";

                                if(File.Exists("friends.txt"))
                                    return Encoding.ASCII.GetBytes(File.ReadAllText("friends.txt"));

                                return Encoding.ASCII.GetBytes("ERROR: friends.txt not found.");
                            }
                        }
                    }

                    lock(m_fixCache)
                    {
                        File.WriteAllLines("friends.txt", m_fixCache.ToArray());
                    }

                    return m_realHandler.Handle(path, GenerateStreamFromString(body), httpRequest, httpResponse);
                }
            }catch(Exception _e)
            {
                m_log.Info("EXAPTION: " + _e.Message);
                return null;
            }
        }
    }
}
